/*
 * Ping.h
 * ping class definition. ping class starts after extern "C" code.
 */
#ifndef __PING_H
#define __PING_H

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/thread.h>
#include <propeller.h>

#ifndef dprintf
#define dprintf
#endif

#ifndef STACK_SIZE 
#define STACK_SIZE 32
#endif

/* stacks for COGs */
static int PingStack[STACK_SIZE];

/* per-thread library variables */
static _thread_state_t PingThreadData;

/* variables that we share between cogs */

/* Ping))) COG variables */
static volatile int PingTime;

static int mpin;

/*
 * Ping))) COG function
 */
static void PingThread(void *arg __attribute__((unused)))
{
    int t0 = 0;
    int t1 = 0;
    int mstime = (CLKFREQ/1000);
    int ustime = (CLKFREQ/1000000);
    int pinmask = (1<<mpin);

    for(;;) {
        
        OUTA |=  pinmask;
        DIRA |=  pinmask;
        waitcnt(5*mstime+CNT);      // trigger time
        OUTA &= ~pinmask;
        DIRA &= ~pinmask;

        waitpeq(pinmask,pinmask);   // wait for rising edge
        t0 = CNT;
        waitpne(pinmask,pinmask);   // wait for falling edge
        t1 = CNT;

        PingTime = ((t1-t0)/ustime)>>1;

        waitcnt(5*mstime+CNT);
    }
}

/* start the threads */

static void startPingThread(void)
{
    int n = _start_cog_thread(PingStack + STACK_SIZE, PingThread, NULL, &PingThreadData);
    dprintf("Ping COG thread %d\n", n);
}

static int getPingTime(void)
{
    return PingTime;
}


#if defined(__cplusplus)
};
#endif


class Ping
{
public:
    Ping(int pin) {
        ::mpin = pin;
    }

    /**
     * start the ping thread
     */
    void start(void)
    {
        ::startPingThread();
    }

    /**
     * get ping reflection time
     */
    int getPingTime(void)
    {
        return ::getPingTime();    
   }
    
    /**
     * convert CLK ticks to inches
     * @param ticks The number of clock ticks from getPingTime
     * @returns distance in inches
     */
    int toInches(int ticks)
    {
        return ((ticks)*1000)/TOIN;
    }

    /**
     * convert CLK ticks to CM centimeters
     * @param ticks The number of clock ticks from getPingTime
     * @returns distance in inches
     */
    int toCM(int ticks)
    {
        return ((ticks)*1000)/TOCM;
    }

#ifdef TEST   
    /**
     * Main program function.
     */
    int test(void)
    {
        /* keep QuickStart TX quiet. */
        DIRA |= (1<<30);
    
        setbuf(stdout, 0);
    
        start();
    
        for(;;) {
            putchar(0);
            dprintf("Ping Test %d\n", toCM(getPingTime()));
            waitcnt(CLKFREQ/10+CNT);
        }
        return 0;
    }
#endif    
 
private:

    enum { TOCM = 29034 };
    enum { TOIN = 73746 };
    
};

#endif
